package cz.vsb.jtestserver.service;

import org.apache.log4j.Logger;
import org.apache.struts.upload.FormFile;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import cz.vsb.jtestserver.util.CompileFile;
import cz.vsb.jtestserver.util.Configuration;
import cz.vsb.jtestserver.util.Database;
import cz.vsb.jtestserver.util.FileUpload;
import cz.vsb.jtestserver.util.TestUserFile;
import cz.vsb.jtestserver.view.PredmetView;
import cz.vsb.jtestserver.view.UlohaView;
import cz.vsb.jtestserver.view.UserView;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.List;

public class TestFileUploadManager {
	static Logger logger = Logger.getLogger(TestFileUploadManager.class);
	
	private UserView userView;

	private PredmetView predmetView;
	
	private String saveTestPath;
	
	private String saveFileTPath;
	
	private String junitPath;

	public TestFileUploadManager(UserView uv, PredmetView pv) {
		userView = uv;
		predmetView = pv;
		saveTestPath = Configuration.getProperty("saveTestPath");
		saveFileTPath = Configuration.getProperty("saveFileTPath"); 
		junitPath = Configuration.getProperty("junitPath");
	}

	public void fileUpload(FormFile formFile, int idUlohy)
			throws FileNotCompiledException, FileNotLoadedException,
			InvalidTestException {

		UlohaManager ulohaM = new UlohaManager(userView);
		UlohaView ulohaView = ulohaM.findUloha(idUlohy);

		String savePath = saveTestPath + java.io.File.separator
				+ predmetView.getZkratka() + java.io.File.separator
				+ ulohaView.getNazevUlohy();
		logger.debug("Cesta pro ukladani: " + savePath);		
		FileUpload fUpload = new FileUpload();
		boolean success = false;
		try {
			success = fUpload.doUpload(formFile.getInputStream(), 
					formFile.getFileName(), savePath);
		} catch (FileNotFoundException fnfe) {
			success = false;
		} catch (IOException ioe) {
			success = false;
		}

		String testPath = saveTestPath + File.separator
				+ predmetView.getZkratka() + java.io.File.separator
				+ ulohaView.getNazevUlohy();
		String fileTPath = saveFileTPath + File.separator
				+ predmetView.getZkratka() + java.io.File.separator
				+ ulohaView.getNazevUlohy();
		savePath = fileTPath + ";" + testPath + ";" + junitPath;
		logger.debug("Cesta pro preklad: " + savePath);

		if (success) {
			CompileFile compileFile = new CompileFile();
			boolean result = compileFile.doCompile(savePath, testPath, testPath
					+ File.separator + formFile.getFileName());
			if (result) {
				ulohaView.setTesty("A");
				ulohaM.updateUlohaTesty(idUlohy, ulohaView.getTesty());

				// jmeno souboru pro porovnani s Jmenem Tridy
				String name = formFile.getFileName();
				String fileName = name.substring(0, name.lastIndexOf('.'));
				//List<String> testy = searchFQName(TestPath, fileName);
				List<String> testy = new ArrayList<String>(1);
				testy.add(fileName);

				TestUserFile tuf = new TestUserFile();
				boolean testResult = tuf.doTest(fileTPath + File.separator,
						testPath + File.separator, junitPath + File.separator,
						testy, userView.getLogin(), ulohaView.getNazevUlohy(),
						userView.getStatus(), 0);
				if (testResult) {
					Method[] metody = null;
					try {
						URL url = new URL("file:///" + testPath + File.separator);
						ClassLoader cl = this.getClass().getClassLoader();
						URLClassLoader ucl = new URLClassLoader(new URL[] { url }, cl);
						Class c = ucl.loadClass(fileName);
						metody = c.getMethods();
					} catch (ClassNotFoundException e) {
						logger.error(e, e);
					} catch (MalformedURLException e) {
						logger.error(e, e);
					}
					Connection con = null;
					try {
						con = Database.getConnection();
						String query = "INSERT INTO testy VALUES(null,?,?,?,?)";
						logger.debug(query);
						PreparedStatement sqlPrep = con.prepareStatement(query);
						sqlPrep.setInt(1, idUlohy);
						sqlPrep.setString(2, fileName);
						sqlPrep.setString(3, getDatum());
						sqlPrep.setString(4, userView.getLogin());
						sqlPrep.executeUpdate();

						query = "SELECT id_testu FROM testy WHERE nazev_testu = '"
							+ fileName + '\'';
						logger.debug(query);
						Statement sql = con.createStatement();
						ResultSet sqlResult = sql.executeQuery(query);
						int testId;
						if (sqlResult.next()) {
							testId = sqlResult.getInt("id_testu");
						} else {
							testId = -1;
						}
                                                sqlResult.close();
                                                sql.close();
                                                con.close();
						if (metody == null) {
							logger.error("Zadne metody nebyly nalezeny");
							return;
						}
						for (int i = 0; i < metody.length; i++) {
							String methodName = metody[i].getName();

							if (methodName.startsWith("test")) {
								query = "INSERT INTO testy_metody VALUES(null,?,?)";
								logger.debug(query);
								sqlPrep = con.prepareStatement(query);
								sqlPrep.setInt(1, testId);
								sqlPrep.setString(2, methodName);
								sqlPrep.executeUpdate();
							}
						}
					} catch (SQLException e) {
						logger.debug(e, e);
					}
				} else {
					logger.warn("Chyba behem testu");
					throw new InvalidTestException("Chyba behem testu");
				}
			} else {
				logger.warn("Soubor nelze prelozit");
				throw new FileNotCompiledException("Soubor nelze zkompilovat");
			}
		} else {
			logger.warn("Soubor nelze nahrat");
			throw new FileNotLoadedException("Soubor nelze nahrat");
		}
	}

	private String getDatum() {
		return DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG)
				.format((new java.util.Date()));
	}
/*
	private List<String> searchFQName(String testPath, String filename) {
		ArrayList<String> result = new ArrayList<String>();
		File f = new File(testPath);
		FilenameFilter fnf = new FilenameFilterAdapter(filename);
		File[] files = f.listFiles(fnf);
		for (File file : files) {
			if (file.isDirectory())
				files = file.listFiles(fnf);
			if (files.length == 1) {
				result.add(file.getPath());
			}
		}
		return result;
	}
	class FilenameFilterAdapter implements FilenameFilter {
		String name = "";

		FilenameFilterAdapter(String name) {
			this.name = name + ".class";
		}

		public boolean accept(File dir, String mask) {
			File f = new File(mask);
			if (f.isDirectory())
				return true;
			return (mask.equalsIgnoreCase(name));
		}
	}
*/

}
