package cz.vsb.jtestserver.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.List;
import java.util.Timer;

import org.apache.log4j.Logger;

import junit.framework.TestFailure;
import junit.framework.TestResult;

/**
 * Trida, ktera se stara o spousteni testu a jejich kontroly
 */
public class TestUserFile {
	static Logger logger = Logger.getLogger(TestUserFile.class);

	private int errorsCount = 0;

	private int failsCount = 0;
	
	private StringBuffer testMessage = new StringBuffer();

	/**
	 * Provadi testy
	 */
	public boolean doTest(String filePath, String testPath, String junitPath,
			List<String> testy, String login, String nazevUlohy, String status,
			int idVysledku) {
		testMessage = new StringBuffer();
		try {
			logger.debug("Provadeni testu");
			ThreadGroup threadgroup = new ThreadGroup("testy");
			Test test = new Test(threadgroup, "test", testy, filePath, testPath,
					junitPath, login, nazevUlohy);
			test.start();
			Timer timer = new Timer();
			timer.schedule(new ProcessTimer(test, timer), 5000L);
			test.join();
			if (!test.isInterrupted()) {
				timer.cancel();
				logger.debug("Vlakno probehlo v poradku");
			} else
				return false;
			if (test.isStopped()) {
				return false;
			}
			logger.debug("Vysledky testu");
			TestResult testresult = test.getTestResult();
			Enumeration<TestFailure> testFailures = testresult.failures();
			Enumeration<TestFailure> testErrors = testresult.errors();
			int i = testresult.runCount();
			logger.debug("Pocet testu: " + i);
			testMessage.append("<b>Pocet testu:</b> ");
			testMessage.append(i);
			testMessage.append("<br>");
			failsCount = test.getFailureCount();
			errorsCount = test.getErrorCount();

			logger.debug("Pocet failu: " + testresult.failureCount());
			logger.debug("Pocet erroru: " + testresult.errorCount());
			logger.debug("Souhrn testu: ");

			if (status.equals("student")) {
				Connection con = null;
				try {
					con = Database.getConnection();
					PreparedStatement sql = con.prepareStatement(
							"UPDATE vysledky SET pocet_eroru = ?, pocet_failu = ?, vysledek = ? WHERE id_vysledku = ?");
					sql.setInt(1, errorsCount);
					sql.setInt(2, failsCount);
					if ((errorsCount == 0) && (failsCount == 0)) {
						sql.setString(3, "1");
					} else {
						sql.setString(3, "0");
					}
					sql.setInt(4, idVysledku);
					logger.debug(sql.toString());
					sql.executeUpdate();
				} catch (SQLException e) {
					logger.error(e, e);
				}
			}
			Enumeration enumeration = test.getEnumTests();
			while (enumeration.hasMoreElements()) {
				String msg = enumeration.nextElement().toString();
				logger.debug(msg);
				testMessage.append(msg);
				testMessage.append("<br>");
			}

			if (testresult.wasSuccessful()) {
				logger.debug("Testy probehly v poradku");
				testMessage.append("Testy probehly v poradku<br>");
			} else {
				logger.debug("Pri prubehu testu doslo k chybe");
				testMessage.append("Pri prubehu testu doslo k chybe: <br>");
				logger.debug("Popis chyby: ");
				testMessage.append("Popis chyby: <br>");
				while (testFailures.hasMoreElements()) {
					String msg = testFailures.nextElement().toString();
					logger.debug(msg);
					testMessage.append(msg);
					testMessage.append("<br>");
				}
				while (testErrors.hasMoreElements()) {
					String msg = testErrors.nextElement().toString();
					logger.debug(msg);
					testMessage.append(msg);
					testMessage.append("<br>");
				}
			}
			return true;
		} catch (Exception e) {
			logger.error(e, e);
			return false;
		}
	}

	public int getPocetErrors() {
		return this.errorsCount;
	}

	public int getPocetFails() {
		return this.failsCount;
	}

	public String getTestMessage() {
		return testMessage.toString();
	}	
}
