package cz.vsb.jtestserver.util;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Trida, ktera generuje XML dokument z vysledku testu
 * 
 * @author David Sousedik
 */
public class XMLDocumentCreator {
	static Logger logger = Logger.getLogger(XMLDocumentCreator.class);

	private Document doc;

	private Element rootElement;

	private Element currentSuite;

	private Hashtable<TestCase, Long> testStarts;

	private OutputStream out = null;

	private long suiteTime = 0;

	private long suitesTime = 0;

	/**
	 * Vytvori zakladni kostru dokumentu
	 */
	public XMLDocumentCreator() {
		testStarts = new Hashtable<TestCase, Long>();
	}

	/**
	 * Vytvoreni instance XML dokumentu
	 */
	private static DocumentBuilder getDocumentBuilder() {
		try {
			return DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			logger.error(e.getMessage());
			throw new ExceptionInInitializerError(e);
		}
	}

	/**
	 * Nastaveni, kam budeme XML dokument zapisovat
	 */
	public void setOutput(OutputStream outputstream) {
		out = outputstream;
	}

	/**
	 * Tvorba rodicovskeho elementu
	 */
	public void startTestSuites(String nazevUlohy, String login, String cas,
			String datum) {
		doc = getDocumentBuilder().newDocument();
		rootElement = doc.createElement("testsuites");
		rootElement.setAttribute("name", nazevUlohy);
		rootElement.setAttribute("login", login);
		rootElement.setAttribute("cas", cas);
		rootElement.setAttribute("datum", datum);
		suiteTime = 0L;
		suitesTime = 0L;
	}

	/**
	 * Ukonceni XML dokumentu
	 */
	public void endTestSuites(int testu, int erroru, int chyb) {
		rootElement.setAttribute("tests", Integer.toString(testu));
		rootElement.setAttribute("errors", Integer.toString(erroru));
		rootElement.setAttribute("failures", Integer.toString(chyb));
		rootElement.setAttribute("time", Double.toString(
				(double) suitesTime / 1000D));
		if (out != null) {
			BufferedWriter bufferedWriter = null;
			try {
				bufferedWriter = new BufferedWriter(new OutputStreamWriter(out,
						"UTF8"));
				bufferedWriter.write(
						"<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
				(new DOMElementWriter()).write(rootElement, bufferedWriter, 0, 
						"	");
				bufferedWriter.flush();
			} catch (IOException ioexception) {
				logger.error(ioexception.getMessage(), ioexception);
			} finally {
				if (bufferedWriter != null) {
					try {
						bufferedWriter.close();
					} catch (IOException ioexception1) {
						logger.error(ioexception1.getMessage());
					}
				}
			}
		}
	}

	/**
	 * Tvorba elementu, reprezentujiciho test
	 */
	public void startTestSuite(TestSuite testsuite) {
		currentSuite = null;
		currentSuite = doc.createElement("testsuite");
	}

	/**
	 * Nastaveni atributu u testu
	 */
	public void endTestSuite(TestSuite testsuite) {
		currentSuite.setAttribute("name", testsuite.getName());
		currentSuite.setAttribute("tests", Integer.toString(
				testsuite.testCount()));
		currentSuite.setAttribute("time", Double.toString((
				double) suiteTime / 1000D));
		suitesTime += suiteTime;
		suiteTime = 0L;
		rootElement.appendChild(currentSuite);
	}

	/**
	 * Start testu
	 */
	public void startTest(TestCase testcase) {
		testStarts.put(testcase, new Long(System.currentTimeMillis()));
	}

	/**
	 * Konec testu
	 */
	public void endTest(TestCase testcase, TestResult testresult) {
		logger.debug("Generuji zpravu o vysledcich testu");
		if (!testStarts.containsKey(testcase))
			startTest(testcase);
		Element element = doc.createElement("testcase");
		element.setAttribute("name", testcase.getName());
		currentSuite.appendChild(element);
		long time = testStarts.get(testcase);
		element.setAttribute("time", Double.toString((double) 
				(System.currentTimeMillis() - time)/ 1000D));
		suiteTime += System.currentTimeMillis() - time;
		Enumeration failures = testresult.failures();
		while (failures.hasMoreElements()) {
			Element fail = doc.createElement("failure");
			element.appendChild(fail);
			fail.setAttribute("message", failures.nextElement()
					.toString());
		}
		Enumeration errors = testresult.errors();
		while (errors.hasMoreElements()) {
			Element error = doc.createElement("error");
			element.appendChild(error);
			error.setAttribute("message", errors.nextElement()
					.toString());			
		}
		logger.debug("Vygenerovano " + element.getChildNodes().getLength()
				+ " chyb a selhani");
	}
}
